/*

Copyright (C) 2007  iPhoneDev Team ( http://iphone.fiveforty.net/wiki/ )

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>

*/

#ifndef SHELL_H
#define SHELL_H

#include <stdio.h>
#include <sys/types.h>

extern "C"
{
#include <readline/history.h>
#include <readline/readline.h>
}

#include "UUTextParser.h"
#include <vector>

#include "MobileDevice.h"
#include "NormalInterface.h"
#include "RecoveryInterface.h"
#include "DFUInterface.h"
#include "RestoreInterface.h"
#include "privateFunctions.h"

//---

enum
{
	kbSE_manifest_none
	,kbSE_manifest_added
	,kbSE_manifest_deleted
	,kbSE_manifest_updated
	,kbSE_manifest_copied
	,kbSE_manifest_backedUp
	,kbSE_manifest_restored
	,kbSE_manifest_setInfo
};

// manifest item

struct bSEManifestItem
{
	int						action;		// kbSE_manifest_...
	string512				path1;
	string512				path2;
};

typedef struct bSEManifestItem bSEManifestItem;

//---

/* Unix version of a hidden file.  Could be different on other systems. */
#define HIDDEN_FILE(fname)	((fname)[0] == '.')

// TODO:  Add windows version of this?
#define PATH_DELIMITER_CHAR '/'
#define PATH_DELIMITER_STRING "/"

#ifndef savestring
#define savestring(x) strcpy ((char *)malloc (1 + strlen (x)), (x))
#endif

#define SHELL_UNKNOWN	-1

enum
{
	SHELL_NORMAL	= 0
	,SHELL_RESTORE	= 1
	,SHELL_RECOVERY	= 2
	,SHELL_DFU		= 3
	
	,SHELL_LAST
};

#define MAX_ARGUMENTS	5

// userspace function exit values
#define SHELL_CONTINUE	-1	// continue reading from interactive shell
#define	SHELL_TERMINATE	0	// terminate application 
#define SHELL_WAIT	1	// wait for device reconnect

typedef int shell_funct (string*, shell_state*);

typedef struct {
	char *name;		// User printable name
	shell_funct *func;	// Function pointer
	char *doc;		// Documentation string
} COMMAND;

// readline
typedef struct shell_state {
	struct afc_connection *afch;
	struct afc_connection *conn;
	struct afc_connection *alt_service;
	struct am_restore_device *restore_dev;
	struct am_recovery_device *recovery_dev;
	struct am_device *dev;
	
	int		shell_mode;
	int		devNo;
	
	string prompt_string;
	string remote_path;
	string local_path;
	
	COMMAND *command_array;
	
	std::vector<bSEManifestItem>	*MF;
	UUTextParser					*manifestTP;
};

struct afc_dirent {
	uint8_t d_namlen;
	char *d_name;
};

#define D_NAMLEN(d)	((d).d_namlen)

// CHK 2007/08/13

#define kForceOutErr	-17173
#define kForceOut		-17172
#define kNoVal			-17171

extern void bSE_pushManifestItem(struct shell_state *sh, int action, const char *path1, const char *path2);

extern void my_cout(const char *str, int val = kNoVal);
extern void my_exit(void);

extern char	gMsgStr[8192];
extern bool gVerbose;
extern char	gLASTVALUE[256];

#define D(string)	my_cout(string, kForceOut) 

//---

typedef int shell_funct (string*, shell_state*);

int exec_line( char *line, struct shell_state *sh );
bool dirExists(afc_connection *hAFC, char *path);
void processRelativePath(string *basePath, string *cdPath);

int shell(struct shell_state *sh);
int sh_help(string *args, struct shell_state *sh);

#endif //SHELL_H

/* -*- mode:c; indent-tabs-mode:nil; c-basic-offset:2; tab-width:2; */
